@@ -30,7 +30,7 @@
 #include "misc.h"
 #include "xdmcp.h"
 
-static const gchar RCSid[]="$Id: display.c,v 1.9 2001/02/04 00:38:43 jirka Exp $";
+static const gchar RCSid[]="$Id: display.c,v 1.10 2001/02/13 23:23:57 jirka Exp $";
 
 /* External vars */
 extern gboolean GdmXdmcp;
@@ -303,18 +303,15 @@ gdm_display_dispose (GdmDisplay *d)
 GdmDisplay *
 gdm_display_lookup (pid_t pid)
 {
-    GSList *list = displays;
-    GdmDisplay *d = NULL;
+    GSList *li;
 
     /* Find slave in display list */
-    while (list && list->data) {
-	d = list->data;
+    for (li = displays; li != NULL; li = li->next) {
+	    GdmDisplay *d = li->data;
 
-	if (pid == d->slavepid) {
-	    return d;
-
-	    list = list->next;
-	}
+	    if (d != NULL &&
+		pid == d->slavepid)
+		    return d;
     }
      
     /* Slave not found */
