@@ -23,12 +23,12 @@
 #include "gdm.h"
 #include "misc.h"
 
-static const gchar RCSid[]="$Id: misc.c,v 1.8 2001/01/22 07:56:13 jirka Exp $";
+static const gchar RCSid[]="$Id: misc.c,v 1.9 2001/01/30 23:23:40 jirka Exp $";
 
 
 /* Configuration option variables */
 extern gchar *GdmPidFile;
-extern gint GdmDebug;
+extern gboolean GdmDebug;
 
 
 /**
