@@ -20,13 +20,21 @@
  * presents a list of them and allows the user to choose one. The
  * selected hostname will be printed on stdout. */
 
+/* This should always be commented before commiting/releasing. When it's
+ * defined the current directory is searched for the glade file, prior to
+ * the system directory.
+ */
+/*#define DOING_DEVELOPMENT 1*/
+
 #include <config.h>
 #include <gnome.h>
+#include <glade/glade.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <dirent.h>
 #include <string.h>
 #include <unistd.h>
+#include <ctype.h>
 #include <gdk/gdkx.h>
 #include <X11/Xmd.h>
 #include <X11/Xdmcp.h>
@@ -45,14 +53,21 @@
 #include "gdmchooser.h"
 #include "gdm.h"
 
-static const gchar RCSid[]="$Id: gdmchooser.c,v 1.9 2001/02/20 02:35:51 jirka Exp $";
+static const gchar RCSid[]="$Id: gdmchooser.c,v 1.11 2001/03/05 22:56:45 lee Exp $";
+
+static const gchar *scanning_message = N_("Please wait: scanning local network for XDMCP-enabled hosts...");
+static const gchar *empty_network = N_("No serving hosts were found.");
+static const gchar *active_network = N_("Choose a host to connect to from the selection below.");
+static gchar *glade_filename = NULL;
 
 static GdmChooserHost *gdm_chooser_host_alloc (gchar *hostname, gchar *description);
 static void gdm_chooser_decode_packet (void);
 static void gdm_chooser_abort (const gchar *format, ...);
 static void gdm_chooser_browser_update (void);
-static void gdm_chooser_xdmcp_init (void);
+static void gdm_chooser_xdmcp_init (gchar **hosts);
 static void gdm_chooser_host_dispose (GdmChooserHost *host);
+static void gdm_chooser_choose_host (gchar *hostname);
+static void gdm_chooser_add_hosts (gchar **hosts);
 
 /* Fixetyfix */
 int XdmcpReallocARRAY8 (ARRAY8Ptr array, int length);
@@ -70,8 +85,10 @@ static XdmAuthRec authlist = {
 
 
 static gint sockfd;
+static XdmcpBuffer bcbuf;
 static XdmcpBuffer querybuf;
 static GSList *bcaddr;
+static GSList *queryaddr;
 
 static gint  GdmIconMaxHeight;
 static gint  GdmIconMaxWidth;
@@ -81,13 +98,11 @@ static gchar *GdmHostIconDir;
 static gchar *GdmHostDefaultIcon;
 static gchar *GdmGtkRC;
 
-static GtkWidget *chooser;
-static GtkWidget *manage;
-static GtkWidget *rescan;
-static GtkWidget *cancel;
+static GladeXML *chooser_app;
+static GtkWidget *chooser, *manage, *rescan, *cancel;
+static GtkWidget *status_label;
 
 static gint maxwidth = 0;
-static guint tid = 0;
 static GIOChannel *channel;
 static GList *hosts = NULL;
 static GdkImlibImage *defhostimg;
@@ -96,7 +111,7 @@ static GdmChooserHost *curhost;
 
 
 static gint 
-gdm_chooser_sort_func(gpointer d1, gpointer d2)
+gdm_chooser_sort_func (gpointer d1, gpointer d2)
 {
     GdmChooserHost *a = d1;
     GdmChooserHost *b = d2;
@@ -119,8 +134,9 @@ gdm_chooser_decode_packet (void)
     gchar *hostname = NULL;
     gchar *status = NULL;
     ARRAY8 auth, host, stat;
+    GdmChooserHost *new_host;
 
-    if (! XdmcpFill (sockfd, &buf, &clnt_sa, &sa_len))
+    if (! XdmcpFill (sockfd, &buf, (XdmcpNetaddr) &clnt_sa, &sa_len))
 	return;
 
     if (! XdmcpReadHeader (&buf, &header))
@@ -152,10 +168,20 @@ gdm_chooser_decode_packet (void)
     /* We can't pipe hostnames larger than this */
     if (strlen (hostname)+1 > PIPE_BUF)
 	goto done;
+
+    new_host = gdm_chooser_host_alloc (hostname, (gchar *) status);
+    if (g_list_find_custom (hosts,
+			    new_host,
+			    (GCompareFunc) gdm_chooser_sort_func)) {
+	    gdm_chooser_host_dispose (new_host);
+	    goto done;
+    }
     
     hosts = g_list_insert_sorted (hosts, 
-				  gdm_chooser_host_alloc (hostname, (gchar *) status),
+				  new_host,
 				  (GCompareFunc) gdm_chooser_sort_func);
+
+    gdm_chooser_browser_update ();
     
  done:
     XdmcpDisposeARRAY8 (&auth);
@@ -220,17 +246,20 @@ gdm_chooser_find_bcaddr (void)
 }
 
 
-static gboolean 
+gboolean 
 gdm_chooser_xdmcp_discover (void)
 {
     struct sockaddr_in sock;
     GSList *bl = bcaddr;
+    GSList *ql = queryaddr;
     struct in_addr *ia;
     GList *hl = hosts;
 
-    gtk_widget_set_sensitive (GTK_WIDGET (chooser), FALSE);
-    gnome_icon_list_freeze (GNOME_ICON_LIST (browser));
+    gtk_widget_set_sensitive (GTK_WIDGET (manage), FALSE);
     gnome_icon_list_clear (GNOME_ICON_LIST (browser));
+    gtk_widget_set_sensitive (GTK_WIDGET (browser), FALSE);
+    gtk_label_set (GTK_LABEL (status_label),
+		   scanning_message);
 
     while (hl) {
 	gdm_chooser_host_dispose ((GdmChooserHost *) hl->data);
@@ -247,19 +276,27 @@ gdm_chooser_xdmcp_discover (void)
     while (bl) {
 	ia = (struct in_addr *) bl->data;
 	sock.sin_addr.s_addr = ia->s_addr; 
-	XdmcpFlush (sockfd, &querybuf, &sock, sizeof (struct sockaddr_in));
+	XdmcpFlush (sockfd, &bcbuf, (XdmcpNetaddr) &sock, sizeof (struct sockaddr_in));
 	bl = bl->next;
     }
 
+    /*
     tid = g_timeout_add (GdmScanTime * 1000, 
 			 (GSourceFunc) gdm_chooser_browser_update, NULL);
+     */
+    while (ql != NULL) {
+	    ia = (struct in_addr *) ql->data;
+	    sock.sin_addr.s_addr = ia->s_addr;
+	    XdmcpFlush (sockfd, &querybuf, (XdmcpNetaddr) &sock, sizeof (struct sockaddr_in));
+	    ql = ql->next;
+    }
 
     return TRUE;
 }
 
 
 static void
-gdm_chooser_xdmcp_init (void)
+gdm_chooser_xdmcp_init (char **hosts)
 {
     static XdmcpHeader header;
     gint sockopts = 1;
@@ -271,15 +308,22 @@ gdm_chooser_xdmcp_init (void)
     if (setsockopt (sockfd, SOL_SOCKET, SO_BROADCAST, (char *) &sockopts, sizeof (sockopts)) < 0)
 	gdm_chooser_abort ("Could not set socket options!");
 
-    gdm_chooser_find_bcaddr();
-
     /* Assemble XDMCP BROADCAST_QUERY packet in static buffer */
     header.opcode  = (CARD16) BROADCAST_QUERY;
     header.length  = 1;
     header.version = XDM_PROTOCOL_VERSION;
+    XdmcpWriteHeader (&bcbuf, &header);
+    XdmcpWriteARRAY8 (&bcbuf, &authlist.authentication);
+
+    /* Assemble XDMCP QUERY packet in static buffer */
+    header.opcode  = (CARD16) QUERY;
+    header.length  = 1;
+    header.version = XDM_PROTOCOL_VERSION;
     XdmcpWriteHeader (&querybuf, &header);
     XdmcpWriteARRAY8 (&querybuf, &authlist.authentication);
 
+    gdm_chooser_add_hosts (hosts);
+
     channel = g_io_channel_unix_new (sockfd);
     g_io_add_watch_full (channel, G_PRIORITY_DEFAULT,
 			G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
@@ -291,7 +335,7 @@ gdm_chooser_xdmcp_init (void)
 }
 
 
-static gboolean
+gboolean
 gdm_chooser_cancel (void)
 {
     closelog();
@@ -301,15 +345,15 @@ gdm_chooser_cancel (void)
 }
 
 
-static gboolean
-gdm_chooser_manage (void)
+gboolean
+gdm_chooser_manage (GtkButton *button, gpointer data)
 {
     if (curhost)
-	g_print ("%s\n", curhost->name);
-
+      gdm_chooser_choose_host (curhost->name);
+   
     closelog();
     gtk_main_quit();
-
+   
     return TRUE;
 }
 
@@ -357,7 +401,7 @@ gdm_chooser_parse_config (void)
 }
 
 
-static gboolean 
+gboolean 
 gdm_chooser_browser_select (GtkWidget *widget, gint selected, GdkEvent *event)
 {
     if (!widget || !event)
@@ -379,7 +423,7 @@ gdm_chooser_browser_select (GtkWidget *widget, gint selected, GdkEvent *event)
 }
 
 
-static gboolean
+gboolean
 gdm_chooser_browser_unselect (GtkWidget *widget, gint selected, GdkEvent *event)
 {
     if (!event) 
@@ -406,7 +450,8 @@ gdm_chooser_browser_update (void)
 {
     GList *list = hosts;
 
-    g_source_remove (tid);
+    gnome_icon_list_freeze (GNOME_ICON_LIST (browser));
+    gnome_icon_list_clear (GNOME_ICON_LIST (browser));
 
     while (list) {
 	GdmChooserHost *host;
@@ -423,27 +468,48 @@ gdm_chooser_browser_update (void)
 
     gnome_icon_list_thaw (GNOME_ICON_LIST (browser));
 
-    gtk_widget_set_sensitive (GTK_WIDGET (chooser), TRUE);
+    if (hosts) {
+      gtk_label_set (GTK_LABEL (glade_xml_get_widget (chooser_app, "status_label")),
+		     active_network);
+    } else {
+      gtk_label_set (GTK_LABEL (glade_xml_get_widget (chooser_app, "status_label")),
+		     empty_network);
+    }
     gtk_widget_set_sensitive (GTK_WIDGET (manage), FALSE);
-    gtk_widget_show_all (GTK_WIDGET (chooser));
+    gtk_widget_set_sensitive (GTK_WIDGET (rescan), TRUE);
+    gtk_widget_set_sensitive (GTK_WIDGET (browser), TRUE);
 }
 
+void
+display_chooser_information (void)
+{
+   glade_xml_new (glade_filename, "gdmchooser_help_dialog");
+}
 
 static void 
 gdm_chooser_gui_init (void)
 {
     GdkWindow *rootwin;
-    GtkWidget *frame;
-    GtkWidget *vbox;
-    GtkWidget *bbox;
-    GtkWidget *buttonpane;
-    GtkWidget *scrollbar;
-    GtkWidget *bframe;
-    GtkAdjustment *adj;
     GtkStyle  *style;
     GdkColor  bbg = { 0, 0xFFFF, 0xFFFF, 0xFFFF };
     struct    stat statbuf;
  
+   
+        /* Look for the glade file in $(datadir)/gdmconfig or, failing that,
+     * look in the current directory.
+	 * Except when doing development, we want the app to use the glade file
+	 * in the same directory, so we can actually make changes easily.
+     */
+	
+#ifndef DOING_DEVELOPMENT
+    glade_filename = gnome_datadir_file("gdm/gdmchooser.glade");
+    if (!glade_filename)
+      {	  
+	  glade_filename = g_strdup("gdmchooser.glade");
+      }
+#else
+	glade_filename = g_strdup("gdmchooser.glade");
+#endif /* DOING_DEVELOPMENT */
     /* Enable theme */
     if (GdmGtkRC)
 	gtk_rc_parse (GdmGtkRC);
@@ -460,22 +526,22 @@ gdm_chooser_gui_init (void)
     rootwin = gdk_window_foreign_new (GDK_ROOT_WINDOW ());
 
     /* Main window */
-    chooser = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-    gtk_container_border_width (GTK_CONTAINER (chooser), 0);
-
-    /* 3D frame for main window */
-    frame = gtk_frame_new (NULL);
-    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
-    gtk_container_border_width (GTK_CONTAINER (frame), 0);
-    gtk_container_add (GTK_CONTAINER (chooser), frame);
-
-    /* Vertical box containing browser box and button pane */
-    vbox = gtk_vbox_new (FALSE, 10);
-    gtk_container_border_width (GTK_CONTAINER (vbox), 10);
-    gtk_container_add (GTK_CONTAINER (frame), vbox);
-
+    g_assert (glade_filename != NULL);
+    chooser_app = glade_xml_new (glade_filename, "gdmchooser_main");
+    glade_xml_signal_autoconnect (chooser_app);
+   
+    chooser = glade_xml_get_widget (chooser_app, "gdmchooser_main");
+    manage = glade_xml_get_widget (chooser_app, "connect_button");
+    rescan = glade_xml_get_widget (chooser_app, "rescan_button");
+    cancel = glade_xml_get_widget (chooser_app, "quit_button");
+    status_label = glade_xml_get_widget (chooser_app, "status_label");
+   
     /* Find background style for browser */
     style = gtk_style_copy (chooser->style);
+   
+    /* FIXME: Forcing the background to be white seems a bit off seeing as
+     * the user might like having a dark theme.
+     */
     style->bg[GTK_STATE_NORMAL] = bbg;
     gtk_widget_push_style (style);
 
@@ -483,95 +549,26 @@ gdm_chooser_gui_init (void)
     if (maxwidth < GdmIconMaxWidth/2)
 	maxwidth = (gint) GdmIconMaxWidth/2;
 
-    browser = GNOME_ICON_LIST (gnome_icon_list_new (maxwidth+20, NULL, FALSE));
+    browser = (GnomeIconList *) glade_xml_get_widget (chooser_app, 
+						     "chooser_iconlist");
     gnome_icon_list_freeze (GNOME_ICON_LIST (browser));
     gnome_icon_list_set_separators (GNOME_ICON_LIST (browser), " /-_.");
-    gnome_icon_list_set_row_spacing (GNOME_ICON_LIST (browser), 2);
-    gnome_icon_list_set_col_spacing (GNOME_ICON_LIST (browser), 2);
+    gnome_icon_list_set_icon_width (GNOME_ICON_LIST (browser), maxwidth+20);
     gnome_icon_list_set_icon_border (GNOME_ICON_LIST (browser), 2);
-    gnome_icon_list_set_text_spacing (GNOME_ICON_LIST (browser), 2);
-    gnome_icon_list_set_selection_mode (GNOME_ICON_LIST (browser), GTK_SELECTION_SINGLE);
-    gtk_signal_connect (GTK_OBJECT (browser), "select_icon",
-			GTK_SIGNAL_FUNC (gdm_chooser_browser_select), NULL);
-    gtk_signal_connect (GTK_OBJECT (browser), "unselect_icon",
-			GTK_SIGNAL_FUNC (gdm_chooser_browser_unselect), NULL);
     gtk_widget_pop_style();
-
-    /* Browser 3D frame */
-    bframe = gtk_frame_new (NULL);
-    gtk_frame_set_shadow_type (GTK_FRAME (bframe), GTK_SHADOW_IN);
-    gtk_container_add (GTK_CONTAINER(bframe), GTK_WIDGET (browser));
-
-    /* Browser scroll bar */
-    adj = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 0.0, 0.0, 0.0, 0.0));
-    scrollbar = gtk_vscrollbar_new (adj);
-    gnome_icon_list_set_vadjustment (browser, adj);
-
-    /* Box containing all browser functionality */
-    bbox = gtk_hbox_new (0, 0);
-    gtk_box_pack_start (GTK_BOX (bbox), GTK_WIDGET (bframe), 1, 1, 0);
-    gtk_box_pack_start (GTK_BOX (bbox), scrollbar, 0, 0, 0);
-    gtk_widget_show_all (GTK_WIDGET (bbox));
-
-    /* Put browser box in main window */
-    gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (bbox), TRUE, TRUE, 0);
-
-    /* Buttons */
-    manage = gtk_button_new_with_label (_("Connect"));
-    gtk_signal_connect (GTK_OBJECT (manage), "clicked",
-			GTK_SIGNAL_FUNC (gdm_chooser_manage), NULL);
-    GTK_WIDGET_SET_FLAGS (GTK_WIDGET (manage), GTK_CAN_DEFAULT);
-    gtk_widget_set_sensitive (GTK_WIDGET (manage), FALSE);
-    gtk_widget_show (GTK_WIDGET (manage));
-
-    rescan = gtk_button_new_with_label (_("Rescan"));
-    gtk_signal_connect(GTK_OBJECT (rescan), "clicked",
-		       GTK_SIGNAL_FUNC (gdm_chooser_xdmcp_discover), NULL);
-    GTK_WIDGET_SET_FLAGS (GTK_WIDGET (rescan), GTK_CAN_DEFAULT);
-    gtk_widget_show (GTK_WIDGET (rescan));
-
-    cancel = gtk_button_new_with_label (_("Cancel"));
-    gtk_signal_connect(GTK_OBJECT (cancel), "clicked",
-		       GTK_SIGNAL_FUNC (gdm_chooser_cancel), NULL);
-    GTK_WIDGET_SET_FLAGS(GTK_WIDGET (cancel), GTK_CAN_DEFAULT);
-    gtk_widget_show (GTK_WIDGET (cancel));
-
-    /* Button pane */
-    buttonpane = gtk_hbox_new(TRUE, 0);
-    gtk_container_set_border_width ( GTK_CONTAINER (buttonpane), 0);
-    gtk_box_pack_start (GTK_BOX (buttonpane), 
-			GTK_WIDGET (manage), TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (buttonpane), 
-			GTK_WIDGET (rescan), TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (buttonpane), 
-			GTK_WIDGET (cancel), TRUE, TRUE, 0);
-    gtk_window_set_default (GTK_WINDOW (chooser), GTK_WIDGET (manage));
-    gtk_widget_show_all (GTK_WIDGET (buttonpane));
-
-    /* Put button pane in main window */
-    gtk_box_pack_end (GTK_BOX (vbox), 
-		      GTK_WIDGET (buttonpane), FALSE, FALSE, 0);
-
-    gtk_widget_show (GTK_WIDGET (vbox));
-    gtk_widget_show (GTK_WIDGET (frame));
-
-    gtk_window_set_policy (GTK_WINDOW (chooser), 1, 1, 1);
-    gtk_window_set_focus (GTK_WINDOW (chooser), GTK_WIDGET (manage));	
-
-    /* Geometry fun */
-    gtk_widget_show_all (GTK_WIDGET (browser));
     gnome_icon_list_thaw (GNOME_ICON_LIST (browser));
+
     gtk_widget_set_usize (GTK_WIDGET (chooser), 
 			  (gint) gdk_screen_width() * 0.4, 
 			  (gint) gdk_screen_height() * 0.6);
 
+    /* FIXME: This should probably obey some user-specified geometry? */
     gtk_window_position (GTK_WINDOW (chooser), GTK_WIN_POS_CENTER);
-    gtk_widget_show_all (GTK_WIDGET (chooser));
 }
 
 
 static void
-gdm_chooser_signals_init(void)
+gdm_chooser_signals_init (void)
 {
     struct sigaction hup;
     sigset_t mask;
@@ -656,12 +653,145 @@ gdm_chooser_host_dispose (GdmChooserHost *host)
     g_free (host);
 }
 
+static gchar *xdm_address = NULL;
+static gchar *client_address = NULL;
+static gint connection_type = 0;
+
+struct poptOption xdm_options [] = {
+  { "xdmaddress", '\0', POPT_ARG_STRING|POPT_ARGFLAG_ONEDASH, &xdm_address, 0,
+    "setting socket for xdm communication", "xdm response socket" },
+  { "clientaddress", '\0', POPT_ARG_STRING|POPT_ARGFLAG_ONEDASH, &client_address, 0, "setting client address to return in response to xdm", "client address" },
+  { "connectionType", '\0', POPT_ARG_INT|POPT_ARGFLAG_ONEDASH, &connection_type, 0, "setting connection type to return in response to xdm", "connection type" },
+  POPT_AUTOHELP
+  { NULL, 0, 0, NULL, 0}
+};
+
+#ifndef ishexdigit
+#define ishexdigit(c) (isdigit(c) || ('a' <= (c) && (c) <= 'f'))
+#endif
+#define HexChar(c)  ('0' <= (c) && (c) <= '9' ? (c) - '0' : (c) - 'a' + 10)
+
+static int
+from_hex (gchar *s, gchar *d, int len)
+{
+  int t;
+  while (len >= 2)
+    {
+      if (!ishexdigit(*s))
+ return 1;
+      t = HexChar (*s) << 4;
+      s++;
+      if (!ishexdigit(*s))
+ return 1;
+      t += HexChar (*s);
+      s++;
+      *d++ = t;
+      len -= 2;
+    }
+  return len;
+}
+
+
+static void
+gdm_chooser_choose_host (gchar *hostname)
+{
+  ARRAY8 tmparr;
+  struct hostent *hentry;
+
+  g_print ("%s\n", curhost->name);
+  if (xdm_address)
+    {
+      struct sockaddr_in in_addr;
+      char xdm_addr[32];
+      char client_addr[32];
+      int fd;
+      char buf[1024];
+      XdmcpBuffer buffer;
+      long family, port, addr;
+      if (strlen (xdm_address) > 64 || from_hex (xdm_address, xdm_addr, strlen (xdm_address)) != 0)
+ gdm_chooser_abort ("gdm_chooser_chooser_host: Invalid xdm address.");
+
+      family = (xdm_addr[0] << 8) | xdm_addr[1];
+      port = (xdm_addr[2] << 8) | xdm_addr[3];
+      addr = (xdm_addr[4] << 24) | (xdm_addr[5] << 16) | (xdm_addr[6] << 8) | xdm_addr[7];
+      in_addr.sin_family = AF_INET;
+      in_addr.sin_port = htons (port);
+      in_addr.sin_addr.s_addr = htonl (addr);
+      if ((fd = socket (PF_INET, SOCK_STREAM, 0)) == -1)
+ gdm_chooser_abort ("gdm_chooser_chooser_host: Couldn't create response socket.");
+
+      if (connect (fd, (struct sockaddr_in *) &in_addr, sizeof (in_addr)) == -1)
+ gdm_chooser_abort ("gdm_chooser_chooser_host: Couldn't connect to xdm.");
+      buffer.data = (BYTE *) buf;
+      buffer.size = sizeof (buf);
+      buffer.pointer = 0;
+      buffer.count = 0;
+
+      if (strlen (client_address) > 64 || from_hex (client_address, client_addr, strlen (client_address)) != 0)
+ gdm_chooser_abort ("gdm_chooser_chooser_host: Invalid client address.");
+      tmparr.data = (BYTE *) client_addr;
+      tmparr.length = strlen (client_address) / 2;
+      XdmcpWriteARRAY8 (&buffer, &tmparr);
+      XdmcpWriteCARD16 (&buffer, (CARD16) connection_type);
+      hentry = gethostbyname (hostname);
+      if (!hentry)
+ gdm_chooser_abort ("gdm_chooser_chooser_host: Couldn't get host entry for %s", hostname);
+
+      tmparr.data = (BYTE *) hentry->h_addr_list[0]; /* XXX */
+      tmparr.length = 4;
+
+      XdmcpWriteARRAY8 (&buffer, &tmparr);
+      write (fd, (char *) buffer.data, buffer.pointer);
+      close (fd);
+    }
+}
+
+static void
+gdm_chooser_add_hosts (gchar **hosts)
+{
+	struct hostent *hostent;
+	struct sockaddr_in qa;
+	struct in_addr *ia;
+	int i;
+
+	for (i = 0; hosts != NULL && hosts[i] != NULL; i++) {
+		char *name = hosts[i];
+
+		if (strcmp (name, "BROADCAST") == 0) {
+			gdm_chooser_find_bcaddr ();
+			continue;
+		}
+		if (strlen (name) == 8 &&
+		    from_hex (name, (char *) &qa.sin_addr, strlen (name)) == 0)
+			qa.sin_family = AF_INET;
+		else if ((qa.sin_addr.s_addr = inet_addr (name)) != -1)
+			qa.sin_family = AF_INET;
+		else if ((hostent = gethostbyname (name)) != NULL
+			 && hostent->h_addrtype == AF_INET
+			 && hostent->h_length == 4) {
+			qa.sin_family = AF_INET;
+			memmove (&qa.sin_addr, hostent->h_addr, 4);
+		} else {
+			continue; /* not a valid address */
+		}
+
+		ia = g_new0 (struct in_addr, 1);
+		ia->s_addr = qa.sin_addr.s_addr;
+		queryaddr = g_slist_append (queryaddr, ia);
+	}
+
+	if (bcaddr == NULL &&
+	    queryaddr == NULL)
+		gdm_chooser_find_bcaddr ();
+}
 
 int 
 main (int argc, char *argv[])
 {
     gchar **fixedargv;
     gint fixedargc, i;
+    gchar **hosts;
+    poptContext ctx;
 
     /* Avoid creating ~gdm/.gnome stuff */
     gnome_do_not_create_directories = TRUE;
@@ -675,7 +805,8 @@ main (int argc, char *argv[])
 	fixedargv[i] = argv[i];
     
     fixedargv[fixedargc-1] = "--disable-sound";
-    gnome_init ("gdmchooser", VERSION, fixedargc, fixedargv);
+    gnome_init_with_popt_table ("gdmchooser", VERSION, fixedargc, fixedargv, xdm_options, 0, &ctx);
+    glade_gnome_init();
     g_free (fixedargv);
 
     bindtextdomain (PACKAGE, GNOMELOCALEDIR);
@@ -686,7 +817,10 @@ main (int argc, char *argv[])
     gdm_chooser_parse_config();
     gdm_chooser_gui_init();
     gdm_chooser_signals_init();
-    gdm_chooser_xdmcp_init();
+
+    hosts = (gchar **) poptGetArgs (ctx);
+    gdm_chooser_xdmcp_init (hosts);
+    poptFreeContext (ctx);
 
     gtk_main();
 
