@@ -1,4 +1,4 @@
-/* 
+/*
  *    GDMconfig, a graphical configurator for the GNOME display manager
  *    Copyright (C) 1999,2000,2001 Lee Mallabone <lee0@callnetuk.com>
  * 
@@ -20,6 +20,7 @@
  * 
  */
 
+#include <config.h>
 #include "gdmconfig.h"
 
 /* This should always be undefined before building ANY kind of production release. */
@@ -39,18 +40,18 @@ gchar *basic_row[1] = { N_("Basic") };
 gchar *expert_row[1] = { N_("Expert") };
 gchar *system_row[1] = { N_("System") };
 
-gchar *desc1 =_("This panel displays the basic options for configuring GDM.\n"
+gchar *desc1 = N_("This panel displays the basic options for configuring GDM.\n"
 				"\n"
 				"If you need finer detail, select 'expert' or 'system setup' from the list above.\n"
 				"\n"
 				"This will display some of the more complex options of GDM that rarely need to be changed.");
-gchar *desc2 =_("This panel displays the more advanced options of GDM.\n"
+gchar *desc2 = N_("This panel displays the more advanced options of GDM.\n"
 				"\n"
 				"Be sure to take care when manipulating the security options, or you could be "
 				"vulnerable to attackers.\n"
 				"\n"
 				"Choose \"System setup\" to change fundamental options in GDM.");
-gchar *desc3 =_("This panel displays GDM's fundamental system settings.\n"
+gchar *desc3 = N_("This panel displays GDM's fundamental system settings.\n"
 				"\n"
 				"You should only change these paths if you really know what you are doing, as an incorrect "
 				"setup could stop your machine from booting properly.\n"
@@ -60,7 +61,7 @@ gchar *desc3 =_("This panel displays GDM's fundamental system settings.\n"
 /* Keep track of X servers, and the selected user level */
 int number_of_servers = 0;
 int selected_server_row = -1;
-int selected_user_level = -1;
+GtkWidget *invisible_notebook = NULL;
 
 /* Main application widget pointer */
 GtkWidget *GDMconfigurator = NULL;
@@ -82,16 +83,36 @@ GtkWidget *get_widget(gchar *widget_name)
 	return ret;
 }
 
+
+/* This function examines a set of commonly named radio buttons and writes out an integer
+ * with gnome_config under 'key'. It looks for max_value number of radio buttons, and writes
+ * the integer with the value of the earliest named radio button it finds.
+ */
+void gdm_radio_write (gchar *radio_base_name, 
+		      gchar *key, 
+		      int max_value)
+{
+   int i = 0;
+   while (i <= max_value) {
+      gchar *widget_name = g_strdup_printf ("%s_%d", radio_base_name, i);
+      if (GTK_TOGGLE_BUTTON (get_widget(widget_name))->active == TRUE) {
+	 gnome_config_set_int(key, i);
+	 g_free (widget_name);
+	 return;
+      }
+      g_free (widget_name);
+      i++;
+   }
+}
+
+
 int
 main (int argc, char *argv[])
 {
-#ifdef ENABLE_NLS
-    bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
+    bindtextdomain (PACKAGE, GNOMELOCALEDIR);
     textdomain (PACKAGE);
-#endif
 
-	/* FIXME! Should replace the number with 'VERSION', but it doesn't seem to pick it up. */
-    gnome_init ("gdmconfig", "2.0.97.1", argc, argv);
+    gnome_init ("gdmconfig", VERSION, argc, argv);
     glade_gnome_init();
 
     /* Make sure the user is root. If not, they shouldn't be messing with 
@@ -108,14 +129,14 @@ main (int argc, char *argv[])
       }
 #endif /* DOING_DEVELOPMENT */
 
-    /* Look for the glade file in $(datadir)/gdmconfig or, failing that,
+    /* Look for the glade file in $(datadir)/gdm or, failing that,
      * look in the current directory.
 	 * Except when doing development, we want the app to use the glade file
 	 * in the same directory, so we can actually make changes easily.
      */
 	
 #ifndef DOING_DEVELOPMENT
-    glade_filename = gnome_datadir_file("gdmconfig/gdmconfig.glade");
+    glade_filename = gnome_datadir_file("gdm/gdmconfig.glade");
     if (!glade_filename)
       {	  
 	  glade_filename = g_strdup("gdmconfig.glade");
@@ -130,24 +151,28 @@ main (int argc, char *argv[])
 	system_notebook = glade_xml_new(glade_filename, "system_notebook");
 	expert_notebook = glade_xml_new(glade_filename, "expert_notebook");
 	
-	gtk_widget_hide(get_widget("basic_notebook"));
-	gtk_widget_hide(get_widget("expert_notebook"));
-	gtk_widget_hide(get_widget("system_notebook"));
-	gtk_box_pack_start(GTK_BOX(get_widget("main_container")),
-					   get_widget("basic_notebook"),
-					   TRUE,TRUE,4);
-	gtk_box_pack_start(GTK_BOX(get_widget("main_container")),
-					   get_widget("expert_notebook"),
-					   TRUE,TRUE,4);
-	gtk_box_pack_start(GTK_BOX(get_widget("main_container")),
-					   get_widget("system_notebook"),
-					   TRUE,TRUE,4);
+   invisible_notebook = gtk_notebook_new();
+   gtk_notebook_set_show_tabs (GTK_NOTEBOOK (invisible_notebook), FALSE);
+   gtk_notebook_set_show_border (GTK_NOTEBOOK (invisible_notebook), FALSE);
+   gtk_notebook_set_tab_border (GTK_NOTEBOOK (invisible_notebook), 0);
+   gtk_notebook_append_page (GTK_NOTEBOOK (invisible_notebook),
+			     get_widget ("basic_notebook"),
+			     NULL);
+   gtk_notebook_append_page (GTK_NOTEBOOK (invisible_notebook),
+			     get_widget ("expert_notebook"),
+			     NULL);
+   gtk_notebook_append_page (GTK_NOTEBOOK (invisible_notebook),
+			     get_widget ("system_notebook"),
+			     NULL);
+   gtk_container_add (GTK_CONTAINER (get_widget ("main_container")),
+		      invisible_notebook);
+   gtk_widget_show (invisible_notebook);
     g_assert(GUI != NULL);
 
     /* Sanity checking */
     GDMconfigurator = get_widget("gdmconfigurator");
     g_assert(GDMconfigurator != NULL);
-    
+
     /* We set most of the user interface NOT wanting signals to get triggered as
      * we do it. Then we hook up the signals, and THEN set a few remaining elements.
      * This ensures sensitivity of some widgets is correct, and that the font picker
@@ -177,41 +202,34 @@ main (int argc, char *argv[])
     gtk_window_set_title(GTK_WINDOW(GDMconfigurator),
 						 _("GNOME Display Manager Configurator"));
     gtk_widget_set_sensitive(GTK_WIDGET(get_widget("apply_button")), FALSE);
+
+    gtk_widget_show (GDMconfigurator);
     
     gtk_main ();
     return 0;
 }
 
 void user_level_row_selected(GtkCList *clist, gint row,
-							 gint column, GdkEvent *event, gpointer data) {
-	/* Keep redraws to a minimum */
-	if (row == selected_user_level)
-	  return;
-	
-	selected_user_level = row;
-	/* This is a bit hacky, but quick and works. */
-	gtk_widget_hide(get_widget("basic_notebook"));
-	gtk_widget_hide(get_widget("expert_notebook"));
-	gtk_widget_hide(get_widget("system_notebook"));
-	switch (row) {
-	 case 0:
-		gtk_widget_show(get_widget("basic_notebook"));
-		gtk_label_set(GTK_LABEL(get_widget("info_label")),
-					  desc1);
-		break;
-	 case 1:
-		gtk_widget_show(get_widget("expert_notebook"));
-		gtk_label_set(GTK_LABEL(get_widget("info_label")),
-					  desc2);
-		break;
-	 case 2:
-		gtk_widget_show(get_widget("system_notebook"));
-		gtk_label_set(GTK_LABEL(get_widget("info_label")),
-					  desc3);
-		break;
-	 default:
-		g_assert_not_reached();
-	}
+			     gint column, GdkEvent *event, gpointer data) {
+   g_assert (row >= 0 && row < 3);
+   gtk_notebook_set_page (GTK_NOTEBOOK (invisible_notebook), row);
+
+   switch (row) {
+   case 0:
+	   gtk_label_set(GTK_LABEL(get_widget("info_label")),
+			 _(desc1));
+	   break;
+   case 1:
+	   gtk_label_set(GTK_LABEL(get_widget("info_label")),
+			 _(desc2));
+	   break;
+   case 2:
+	   gtk_label_set(GTK_LABEL(get_widget("info_label")),
+			 _(desc3));
+	   break;
+   default:
+	   g_assert_not_reached();
+   }
 }
 
 void show_about_box(void) {
@@ -258,8 +276,7 @@ gdm_config_parse_most (void)
     /* Fill the widgets in Security tab */
     gdm_toggle_set("allow_root", gnome_config_get_bool(GDM_KEY_ALLOWROOT));
     gdm_toggle_set("kill_init_clients", gnome_config_get_bool(GDM_KEY_KILLIC));
-    /* FIXME: see comment on _set for relax_perms */
-    gdm_toggle_set("relax_perms", gnome_config_get_int(GDM_KEY_RELAXPERM));
+    gdm_radio_set ("relax_perms", gnome_config_get_int(GDM_KEY_RELAXPERM));
     gdm_toggle_set("verbose_auth", gnome_config_get_bool(GDM_KEY_VERBAUTH));
 
     gdm_entry_set("gdm_runs_as_user", gnome_config_get_string (GDM_KEY_USER));
@@ -458,8 +475,7 @@ write_new_config_file                  (GtkButton *button,
     /* Write out the widget contents of the Security tab */
     gdm_toggle_write("allow_root", GDM_KEY_ALLOWROOT);
     gdm_toggle_write("kill_init_clients", GDM_KEY_KILLIC);
-    /* FIXME: This should allow 0, 1, 2 levels, it should really be an enum! */
-    gdm_toggle_write_int("relax_perms", GDM_KEY_RELAXPERM);
+    gdm_radio_write ("relax_perms", GDM_KEY_RELAXPERM, 2);
     gdm_toggle_write("verbose_auth", GDM_KEY_VERBAUTH);
 
     gdm_entry_write("gdm_runs_as_user", GDM_KEY_USER);
@@ -563,7 +579,7 @@ void
 open_help_page                         (GtkButton *button,
                                         gpointer         user_data)
 {
-	gnome_error_dialog("No help has been written yet!\nMail docs@gnome.org if you wish to volunteer.");
+	gnome_warning_dialog("Documentation is being written but is not yet finished.\nPlease be patient.");
 	/* FIXME: ! */
 }
 
