@@ -40,9 +40,8 @@
 #endif
 #include "gdmlogin.h"
 #include "gdm.h"
-#include "filecheck.h"
 
-static const gchar RCSid[]="$Id: gdmlogin.c,v 1.32 2001/02/21 06:43:32 jirka Exp $";
+static const gchar RCSid[]="$Id: gdmlogin.c,v 1.35 2001/03/12 04:24:07 jirka Exp $";
 
 static gboolean GdmBrowser;
 static gboolean GdmDebug;
@@ -74,7 +73,7 @@ static GtkWidget *login;
 static GtkWidget *label;
 static GtkWidget *entry;
 static GtkWidget *msg;
-static gboolean first_message = TRUE;
+static gboolean require_quater = FALSE;
 static GtkWidget *win;
 static GtkWidget *sessmenu;
 static GtkWidget *langmenu;
@@ -89,7 +88,6 @@ static GdkImlibImage *defface;
 static GSList *sessions = NULL;
 static GSList *languages = NULL;
 static GList *users = NULL;
-static GSList *exclude = NULL;
 
 static gchar *defsess = NULL;
 static gchar *cursess = NULL;
@@ -160,9 +158,12 @@ gdm_greeter_chld (int sig)
 static void
 kill_background (void)
 {
-	if (backgroundpid != 0) {
-		kill (backgroundpid, SIGTERM);
-		backgroundpid = 0;
+	pid_t bpid = backgroundpid;
+
+	backgroundpid = 0;
+
+	if (bpid != 0) {
+		kill (bpid, SIGTERM);
 	}
 }
 
@@ -311,8 +312,8 @@ gdm_login_iconify_handler (GtkWidget *widget, gpointer data)
 
     gtk_widget_show (GTK_WIDGET (win));
 
-    rw = gdk_screen_width();
-    rh = gdk_screen_height();
+    rw = screen.width;
+    rh = screen.height;
 
     set_screen_pos (GTK_WIDGET (win), rw-iw, rh-ih);
 }
@@ -470,6 +471,7 @@ gdm_center_window (GtkWindow *cw)
 
 	x = screen.x + (screen.width - req.width)/2;
 	y = screen.y + (screen.height - req.height)/2;	
+
  	gtk_widget_set_uposition (GTK_WIDGET (cw), x, y);	
 }
 
@@ -486,7 +488,7 @@ gdm_login_query (const gchar *msg)
 	    
     gtk_window_set_modal (GTK_WINDOW (req), TRUE);
     gdm_center_window (GTK_WINDOW (req));
-    return (!gnome_dialog_run (GNOME_DIALOG(req)));
+    return (!gnome_dialog_run (GNOME_DIALOG (req)));
 }
 
 
@@ -732,9 +734,9 @@ dance (gpointer data)
 	static int height = -1;
 
 	if (width == -1)
-		width = gdk_screen_width ();
+		width = screen.width;
 	if (height == -1)
-		height = gdk_screen_height ();
+		height = screen.height;
 
 	if (login == NULL ||
 	    login->window == NULL) {
@@ -748,8 +750,8 @@ dance (gpointer data)
 	t1 += 0.03 + (rand () % 10) / 500.0;
 	t2 += 0.03 + (rand () % 10) / 500.0;
 
-	x = (width / 2) + (width / 5) * xm;
-	y = (height / 2) + (height / 5) * ym;
+	x = screen.x + (width / 2) + (width / 5) * xm;
+	y = screen.y + (height / 2) + (height / 5) * ym;
 
 	set_screen_pos (login,
 			x - login->allocation.width / 2,
@@ -762,7 +764,9 @@ static gboolean
 evil (const char *user)
 {
 	static gboolean old_lock;
+
 	if (dance_handler == 0 &&
+	    /* do not translate */
 	    strcmp (user, "Start Dancing") == 0) {
 		setup_cursor (GDK_UMBRELLA);
 		dance_handler = gtk_timeout_add (50, dance, NULL);
@@ -771,24 +775,35 @@ evil (const char *user)
 		gtk_entry_set_text (GTK_ENTRY (entry), "");
 		return TRUE;
 	} else if (dance_handler != 0 &&
+		   /* do not translate */
 		   strcmp (user, "Stop Dancing") == 0) {
 		setup_cursor (GDK_LEFT_PTR);
 		gtk_timeout_remove (dance_handler);
 		dance_handler = 0;
 		GdmLockPosition = old_lock;
+		gdm_center_window (GTK_WINDOW (login));
 		gtk_entry_set_text (GTK_ENTRY (entry), "");
 		return TRUE;
+				 /* do not translate */
 	} else if (strcmp (user, "Gimme Random Cursor") == 0) {
 		setup_cursor (((rand () >> 3) % (GDK_LAST_CURSOR/2)) * 2);
 		gtk_entry_set_text (GTK_ENTRY (entry), "");
+		return TRUE;
+				 /* do not translate */
+	} else if (strcmp (user, "Require Quater") == 0) {
+		require_quater = TRUE;
+		gtk_entry_set_text (GTK_ENTRY (entry), "");
 		return TRUE;
 	}
+
 	return FALSE;
 }
 
 static gboolean
 gdm_login_entry_handler (GtkWidget *widget, GdkEventKey *event)
 {
+    static gboolean first_return = TRUE;
+
     if (!event)
 	return(TRUE);
 
@@ -818,6 +833,12 @@ gdm_login_entry_handler (GtkWidget *widget, GdkEventKey *event)
 	    }
 	}
 
+	/* somewhat ugly thing to clear the initial message */
+	if (first_return) {
+	       first_return = FALSE;
+	       gtk_label_set (GTK_LABEL (msg), "");
+	}
+
 	g_print ("%c%s\n", STX, gtk_entry_get_text (GTK_ENTRY (entry)));
 	break;
 
@@ -1057,13 +1078,13 @@ gdm_login_language_init (GtkWidget *menu)
     fclose (langlist);
 }
 
-
 static gboolean
 gdm_login_ctrl_handler (GIOChannel *source, GIOCondition cond, gint fd)
 {
     gchar buf[PIPE_SIZE];
     gint len;
     gint i, x, y;
+    static gboolean replace_msg = TRUE;
 
     /* If this is not incoming i/o then return */
     if (cond != G_IO_IN) 
@@ -1092,14 +1113,6 @@ gdm_login_ctrl_handler (GIOChannel *source, GIOCondition cond, gint fd)
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 	buf[len-1] = '\0';
 
-	/* Turn off the message whenever the prompt changes,
-	   this is sort of a hack. Also, don't turn it off
-	   the first time. Yeah I know.  */
-	if (first_message)
-	  first_message = FALSE;
-	else
-	  gtk_label_set (GTK_LABEL(msg), "");
-
 	gtk_label_set (GTK_LABEL (label), buf);
 	gtk_widget_show (GTK_WIDGET (label));
 	gtk_entry_set_text (GTK_ENTRY (entry), "");
@@ -1107,20 +1120,15 @@ gdm_login_ctrl_handler (GIOChannel *source, GIOCondition cond, gint fd)
 	gtk_widget_set_sensitive (entry, TRUE);
 	gtk_widget_grab_focus (entry);	
 	gtk_widget_show (entry);
+
+	/* replace rapther then append next message string */
+	replace_msg = TRUE;
 	break;
 
     case GDM_NOECHO:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 	buf[len-1] = '\0';
 
-	/* Turn off the message whenever the prompt changes,
-	   this is sort of a hack. Also, don't turn it off
-	   the first time. Yeah I know.  */
-	if (first_message)
-	  first_message = FALSE;
-	else
-	  gtk_label_set (GTK_LABEL(msg), "");
-
 	gtk_label_set (GTK_LABEL(label), buf);
 	gtk_widget_show (GTK_WIDGET (label));
 	gtk_entry_set_text (GTK_ENTRY (entry), "");
@@ -1128,12 +1136,33 @@ gdm_login_ctrl_handler (GIOChannel *source, GIOCondition cond, gint fd)
 	gtk_widget_set_sensitive (entry, TRUE);
 	gtk_widget_grab_focus (entry);	
 	gtk_widget_show (entry);
+
+	/* replace rapther then append next message string */
+	replace_msg = TRUE;
 	break;
 
     case GDM_MSGERR:
 	g_io_channel_read (source, buf, PIPE_SIZE-1, &len);
 	buf[len-1] = '\0';
-	gtk_label_set (GTK_LABEL (msg), buf);
+
+	/* HAAAAAAACK.  Sometimes pam send many many messages, SO
+	 * we try to collect them until the next prompt or reset or
+	 * whatnot */
+	if ( ! replace_msg) {
+		char *oldtext;
+		gtk_label_get (GTK_LABEL (msg), &oldtext);
+		if (oldtext != NULL && oldtext[0] != '\0') {
+			char *newtext;
+			newtext = g_strdup_printf ("%s\n%s", oldtext, buf);
+			gtk_label_set (GTK_LABEL (msg), newtext);
+		} else {
+			gtk_label_set (GTK_LABEL (msg), buf);
+		}
+	} else {
+		gtk_label_set (GTK_LABEL (msg), buf);
+	}
+	replace_msg = FALSE;
+
 	gtk_widget_show (GTK_WIDGET (msg));
 	g_print ("%c\n", STX);
 
@@ -1206,12 +1235,34 @@ gdm_login_ctrl_handler (GIOChannel *source, GIOCondition cond, gint fd)
 	if (GdmBrowser)
 	    gtk_widget_set_sensitive (GTK_WIDGET (browser), TRUE);
 
+	gtk_label_set (GTK_LABEL (msg), buf);
+	gtk_widget_show (GTK_WIDGET (msg));
+
 	g_print ("%c\n", STX);
 	break;
 
     case GDM_QUIT:
-        kill_background ();
-	exit (EXIT_SUCCESS);
+	g_io_channel_read (source, buf, PIPE_SIZE-1, &len); /* Empty */
+
+	if (require_quater) {
+		GtkWidget *d;
+
+		/* translators:  This is a nice and evil eggie text, translate
+		 * to your favourite currency */
+		d = gnome_message_box_new (_("Please insert 25 cents "
+					     "to log in."),
+					   GNOME_MESSAGE_BOX_INFO,
+					   GNOME_STOCK_BUTTON_OK,
+					   NULL);
+		gtk_window_set_modal (GTK_WINDOW (d), TRUE);
+		gdm_center_window (GTK_WINDOW (d));
+
+		gnome_dialog_run (GNOME_DIALOG (d));
+	}
+
+	kill_background ();
+
+	g_print ("%c\n", STX);
 	break;
 	
     default:
@@ -1704,7 +1755,8 @@ gdm_login_gui_init (void)
 			      (gint) screen.height * 0.25);
     }
 
-    if (GdmLogo && !access (GdmLogo, R_OK)) {
+    if (GdmLogo &&
+	access (GdmLogo, R_OK) == 0) {
 	GtkWidget *logo;
 
 	logoframe = gtk_frame_new (NULL);
@@ -1761,6 +1813,7 @@ gdm_login_gui_init (void)
     gtk_misc_set_padding (GTK_MISC (label), 10, 5);
     
     entry = gtk_entry_new_with_max_length (32);
+    gtk_widget_set_usize (entry, 120, -1);
     gtk_widget_ref (entry);
     gtk_object_set_data_full (GTK_OBJECT (login), "entry", entry,
 			      (GtkDestroyNotify) gtk_widget_unref);
@@ -1784,7 +1837,6 @@ gdm_login_gui_init (void)
 		      (GtkAttachOptions) (GTK_FILL), 0, 10);
         
     msg = gtk_label_new (_("Please enter your login"));
-    first_message = TRUE;
     gtk_widget_set_name(msg, "Message");
     gtk_widget_ref (msg);
     gtk_object_set_data_full (GTK_OBJECT (login), "msg", msg,
@@ -1856,12 +1908,11 @@ gdm_login_sort_func (gpointer d1, gpointer d2)
 
 
 static GdmLoginUser * 
-gdm_login_user_alloc (gchar *logname, uid_t uid, gchar *homedir)
+gdm_login_user_alloc (const gchar *logname, uid_t uid, const gchar *homedir)
 {
     GdmLoginUser *user;
-    gboolean fileok;
-    gchar *gnomedir = NULL;
     GdkImlibImage *img = NULL;
+    gchar *filename;
 
     user = g_new0 (GdmLoginUser, 1);
 
@@ -1872,30 +1923,21 @@ gdm_login_user_alloc (gchar *logname, uid_t uid, gchar *homedir)
     user->login = g_strdup (logname);
     user->homedir = g_strdup (homedir);
 
-    gnomedir = g_strconcat (homedir, "/.gnome", NULL);
+    filename = g_strconcat (homedir, "/.gnome/photo", NULL);
 
-    fileok = gdm_file_check ("gdm_login_user_alloc", uid, gnomedir, "photo", 
-			     FALSE, GdmUserMaxFile, GdmRelaxPerms);
-    
-    if (fileok) {
-	gchar *filename;
-	
-	filename = g_strconcat (gnomedir, "/photo", NULL);
-	img = gdk_imlib_load_image (filename);
-	g_free (filename);
-    }
-    else {
-	gchar *filename;
-	
-	filename = g_strconcat (GdmGlobalFaceDir, "/", logname, NULL);
-	
-	if (access (filename, R_OK) == 0)
+    img = NULL;
+    if (access (filename, R_OK) == 0) {
 	    img = gdk_imlib_load_image (filename);
+    } else {
+	    g_free (filename);
+	    filename = g_strconcat (GdmGlobalFaceDir, "/", logname, NULL);
 	
-	g_free (filename);
+	    if (access (filename, R_OK) == 0)
+		    img = gdk_imlib_load_image (filename);
+
     }
     
-    g_free (gnomedir);
+    g_free (filename);
     
     if(img) {
 	gint w, h;
@@ -1923,30 +1965,38 @@ gdm_login_user_alloc (gchar *logname, uid_t uid, gchar *homedir)
 }
 
 
-static gint
+static gboolean
 gdm_login_check_exclude (struct passwd *pwent)
 {
-    const char * const lockout_passes[] = { "*", "!!", NULL };
-    GSList *list = exclude;
-    gint i;
+	const char * const lockout_passes[] = { "*", "!!", NULL };
+	gint i;
 
-    for (i=0 ; lockout_passes[i] ; i++) 
-	if(strcmp (lockout_passes[i], pwent->pw_passwd) == 0)
-	    return (TRUE);
- 
-     while (list && list->data) {
-	 if (g_strcasecmp (pwent->pw_name, (gchar *) list->data) == 0)
-	     return (TRUE);
+	for (i=0 ; lockout_passes[i] != NULL ; i++)  {
+		if (strcmp (lockout_passes[i], pwent->pw_passwd) == 0) {
+			return TRUE;
+		}
+	}
 
-	 list = list->next;
-     }
+	if (GdmExclude != NULL &&
+	    GdmExclude[0] != '\0') {
+		char **excludes;
+		excludes = g_strsplit (GdmExclude, ",", 0);
 
-     return (FALSE);
+		for (i=0 ; excludes[i] != NULL ; i++)  {
+			if (g_strcasecmp (excludes[i], pwent->pw_name) == 0) {
+				g_strfreev (excludes);
+				return TRUE;
+			}
+		}
+		g_strfreev (excludes);
+	}
+
+	return FALSE;
 }
 
 
-static gint
-gdm_login_check_shell (gchar *usersh)
+static gboolean
+gdm_login_check_shell (const gchar *usersh)
 {
     gint found = 0;
     gchar *csh;
@@ -1970,20 +2020,16 @@ gdm_login_users_init (void)
     struct passwd *pwent;
 
     if (access (GdmDefaultFace, R_OK)) {
-	syslog (LOG_WARNING, _("Can't open DefaultImage: %s. Suspending face browser!"), GdmDefaultFace);
-	GdmBrowser = FALSE;
-	return;
+	    syslog (LOG_WARNING,
+		    _("Can't open DefaultImage: %s. Suspending face browser!"),
+		    GdmDefaultFace);
+	    GdmBrowser = FALSE;
+	    return;
+    } else  {
+	    defface = gdk_imlib_load_image (GdmDefaultFace);
     }
-    else 
-	defface = gdk_imlib_load_image (GdmDefaultFace);
 
-    if (GdmExclude) {
-        gchar *s = strtok (GdmExclude, ",");
-        exclude = g_slist_append (exclude, g_strdup (s));
-
-        while ((s = strtok (NULL, ","))) 
-	    exclude = g_slist_append (exclude, g_strdup (s));
-    }
+    setpwent ();
 
     pwent = getpwent();
 	
@@ -2141,7 +2187,7 @@ main (int argc, char *argv[])
     ctrlch = g_io_channel_unix_new (STDIN_FILENO);
     g_io_channel_init (ctrlch);
     g_io_add_watch (ctrlch, 
-		    G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
+		    G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
 		    (GIOFunc) gdm_login_ctrl_handler,
 		    NULL);
     g_io_channel_unref (ctrlch);
